import pickle as pkl
import numpy as np


def load_limits(filename="limits.pkl"):
    return pkl.load(open(filename, "rb"))


def load_rmse_file(_filename):
    # Routine that reads a netcdf file
    return pkl.load(open(_filename, "rb"))


def read_arguments():
    # Using optparse library to get command line options and arguments
    from optparse import OptionParser

    # Reading the command line arguments and
    # Creating the parser
    parser = OptionParser()
    # Adding options
    # Option for generating simplified output
    # parser.add_option("-r", help="Reference File", dest="reference", type="string", default="year_01")
    parser.add_option("-f", help="File to evaluate", dest="to_evaluate", type="string")

    # Parse arguments and options
    (options, args) = parser.parse_args()
    print(options)

    _evaluated_filename_pattern = options.to_evaluate

    return _evaluated_filename_pattern


def evaluate_dictionaries(simulation_rmse, limit_file):
    # For each grid (In this case probably only grid_T)
    for grid in simulation_rmse.keys():
        for variable in simulation_rmse[grid].keys():
            try:
                failed = (simulation_rmse[grid][variable] > limit_file[grid][variable][1]).any() \
                         or (simulation_rmse[grid][variable] < limit_file[grid][variable][0]).any()
                if failed:
                    return False
            except KeyError:
                pass
                # print("Variable %s is not found in the limit file" % variable)
    return True


def evaluate_fitness_from_dictionaries_old(simulation_rmse, limit_file):
    fitness = 0.
    for grid in simulation_rmse.keys():
        for variable in simulation_rmse[grid].keys():
            try:
                top_error = simulation_rmse[grid][variable] - limit_file[grid][variable][1]
                top_error = np.sum(top_error[simulation_rmse[grid][variable] > limit_file[grid][variable][1]])
                bottom_error = limit_file[grid][variable][0] - simulation_rmse[grid][variable]
                bottom_error = np.sum(bottom_error[simulation_rmse[grid][variable] < limit_file[grid][variable][0]])
                if top_error:
                    fitness += top_error
                if bottom_error:
                    fitness += bottom_error
            except KeyError:
                print("Variable %s is not found in the limit file" % variable)
    return fitness


def evaluate_fitness_from_dictionaries(simulation_rmse, limit_file):
    fitness = 0.
    for grid in simulation_rmse.keys():
        for variable in simulation_rmse[grid].keys():
            try:
                ensemble_mean = limit_file[grid][variable][2]
                distance_from_ensemble_mean = simulation_rmse[grid][variable] - ensemble_mean
                fitness += np.linalg.norm(distance_from_ensemble_mean)
            except KeyError:
                print("Variable %s is not found in the limit file" % variable)
    print(fitness)
    return fitness
